/***************************************************************************
 *                                                                         *
 *       The information  contained herein is  PROPRIETARY  and            *
 *          CONFIDENTIAL to NewMonics Inc.  Patents pending.               *
 *                                                                         *
 *       Copyright (C) 1997 NewMonics Inc., All Rights reserved.           *
 *                                                                         *
 ***************************************************************************/

//
// Contributors to this demonstration:
//
// Dr. Kelvin Nilsen
// Dr. Jim Lathrop
// Paul Thompson
// Simanta Mitra
// Jason Britton
// Steve Lee
//

package demo.BlackJack;

import java.lang.String;

import COM.newmonics.picoPERC.MM;

public class BlackJack
{
  //
  // note that these objects can be swapped out.  for example, ASCIIViaJava
  // can be replaced with GUI for a different output display
  //
  // as it stands it is a compile time option to switch these, but this can
  // easily be changed to a runtime option by using a factory to create the
  // IO object, and then having the factory load the appropriate class
  // at runtime based on something such as a configuration file value
  //
  static Input  input      = new KeyboardViaJava();
  static Output output     = new ASCIIViaJava();
  static RandGen generator = new RandGenViaJava();
  
  private Hand dealer = new Hand(true);
  private Hand player = new Hand(false);
  private int house_money = 100000;
  private int player_money = 100;
  private int bet;
  
  public static void main(String[] args)
  {
    new BlackJack().go();
  }
  
  //
  // the heart of the game.
  //
  public void go()
  {
    Deck deck = new Deck();
    boolean done_playing = false;
    boolean player_standing = false;
    boolean dealer_standing = false;
    boolean dealer_over = false;
    boolean player_over = false;
    Card c;
    char hit;
    
    //
    // Keep at it until the deck is used up
    //
    while (! done_playing)
    {
      startGame();
      
      //
      // read in a number from the user. If its badly formed,
      // politely ask again
      //
      bet = input.getBet();
      if (bet > player_money)
      {
        output.print("You dont have that much\n");
        continue;
      }
      
      output.print("You bet: ");
      output.print(bet);
      output.print("\n");
      player_money -= bet;
      
      //
      // deal out the initial hand
      //
      for (int i = 0; i < 2; ++i)
      {
        player.addCard(deck.dealOne());
        dealer.addCard(deck.dealOne());
      }
      showDealer("D> ");
      showPlayer("P> ");
      
      if (! blackJack())
      {
        //
        // Hit the player until he stands
        //
        while (! player_standing)
        {
          //output.print("hit ('h') or stay ('s') ?\n");
          hit = input.getAction();
          if (hit == 'h')
          {
            player.addCard(deck.dealOne());
            showPlayer("P> ");
            if (player.calcValue() > 21)
            {
              showPlayer("P Over> ");
              dealerWins();
              player_over = true;
              break;
            }
          } 
          else // 's' or anything else
          {
            player_standing = true;
          }
        }
        
        //
        // player is done; dealer reveals his cards, and draws to
        // completion
        // if player had gone over, all has been revealed
        //
        if (! player_over)
        {
          dealer.reveal();
          showDealer("D> ");
          
          while (! dealer_standing)
          {
            //
            // Ok. now it's the dealers turn
            //
            if (dealer.calcValue() <= 16)
            {
              dealer.addCard(deck.dealOne());
              if (dealer.calcValue() > 21)
              {
                showDealer("D Over> ");
                playerWins();
                dealer_over = true;
                break;
              }
            }
            else
            {
              dealer_standing = true;
            }
            
            showDealer("D> ");
          }
        }
        
        //
        // indicate winners and losers, if they stood and waited
        //
        if (! player_over)
        {
          if (! dealer_over)
          {
            int d_val = dealer.calcValue();
            int p_val = player.calcValue();
            
            if (d_val > p_val)
            {
              dealerWins();
            }
            else if (p_val > d_val)
            {
              playerWins();
            }
            else
            {
              pushOff();
            }
          }
        }
      }
      
      //
      // get ready for a new hand!  wait till they hit enter first
      //
      //output.print("hit enter to play another game\n");
      input.getAction();
      player_standing = false;
      dealer_standing = false;
      player_over = false;
      dealer_over = false;
      player.empty();
      dealer.empty();
      deck.newGame();
      dealer.hide();
    }
  }

  protected void startGame()
  {
    output.print("[");
    output.print(player_money);
    output.print("]Bet: \n");
    output.clearCardArea();
  }

  //
  // displays the dealer's hand, prefixed with something
  //
  protected void showDealer(String prefix)
  {
    String s;
  
    output.print(prefix);
    output.print(s = dealer.toString());
    output.print("\n");
    dealer.displayCards(150);
    MM.delete(s);
  }

  //
  // displays the player's hand, prefixed with something
  //
  private void showPlayer(String prefix)
  {
    String s;
  
    output.print(prefix);
    output.print(s = player.toString());
    output.print("\n");
    player.displayCards(320);
    MM.delete(s);
  }

  //
  // can account for winnings later
  //
  private void playerWins()
  {
    showPlayer("P Wins> ");
    dealer.reveal();
    showDealer("D Loses> ");
    player_money += 2 * bet;
  }

  //
  // if the dealer wins display it properly
  //
  private void dealerWins()
  {
    dealer.reveal();
    showDealer("D Wins> ");
    showPlayer("P Loses> ");
    house_money += bet;
  }

  //
  // if its a tie
  //
  private void pushOff()
  {
    dealer.reveal();
    showDealer("D Ties> ");
    showPlayer("P Ties> ");
    player_money += bet;
  }

  //
  // see if anyone had a natural 21 right off, and handle it
  //
  private boolean blackJack()
  {
    //
    // first check for naturals
    //
    if (player.calcValue() == 21)
    {
      if (dealer.calcValue() == 21)
      {
        //
        // Player keeps his bet
        //
        pushOff();
        return true;
      }
      else
      {
        //
        // Player gets 3/2 return on bet
        //
        output.print("P BlackJack!\n");
        player_money += (bet + ((bet * 3) / 2));
        return true;
      }
    }
    else if (dealer.calcValue() == 21)
    {
      //
      // Dealer takes players bet
      //
      output.print("D BlackJack!\n");
      house_money += bet;
      return true;
    }
  
    return false;
  }
}
