import java.awt.Color;
import java.awt.Component;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Rectangle;

/*
 *
 * ComponentList
 *
 * This lightweight component displays the components of a Container in
 * a list. Note: this is not a container itself! You cannot (and do not
 * need to) add/remove things from this component. Calling its repaint()
 * method will refresh it.
 *
 */
public class ComponentList extends Component
{
	Container watch;				// container to watch.
	private int currentLocation;	// current location (y coordinate)
	String title;					// (optional) title
	Color titleColor;				// title color

	//
	// Constructors.
	//
	public ComponentList(Container c)
	{
		this(c, null, null);
	}
	public ComponentList(Container c, String t, Color color)
	{
		watch = c; title = t; titleColor = color;
	}

	//
	// Paint method.
	//
	/* synchronized, but we should never be running the same paint in two different threads */
	public synchronized void paint(Graphics g)
	{
		Component[] c = watch.getComponents();	// this makes a copy and is synch, so is safe.
		currentLocation = 0;
		Rectangle clipRect = g.getClipBounds();
		int max = clipRect.height + clipRect.y;
		if (title != null) {
			g.setColor(titleColor);
			write(g, title, max);
		}
		for (int i=0; i<c.length; ++i) {
			if (write(g, c[i].getName(), max)) break;
		}
		super.paint(g);
	}

	//
	// Writes the string s at the current location on graphics context g.
	// Returns whether the new current location is greater than max_y.
	//
	private boolean write(Graphics g, String s, int max_y)
	{
		currentLocation += g.getFontMetrics().getHeight();
		//System.out.println(currentLocation+": "+s);
		if (s != null) g.drawString(s, 0, currentLocation);
		return (currentLocation >= max_y);
	}

	//
	// Size methods.
	//
	public Dimension getPreferredSize()
	{
		return new Dimension(250, 250);
	}

}
