/***************************************************************************
 *                                                                         *
 *       The information  contained herein is  PROPRIETARY  and            *
 *          CONFIDENTIAL to NewMonics Inc.  Patents pending.               *
 *                                                                         *
 *       Copyright (C) 1997 NewMonics Inc., All Rights reserved.           *
 *                                                                         *
 ***************************************************************************/

package demo.BlackJack;

public class Deck 
{
  private static final int TOTAL_CARDS = 52;
  private static final char[] suits = { 'H', 'D', 'C', 'S' };
  private static final char[] faces =
      { '2', '3', '4', '5', '6', '7', '8', '9', 'T', 'A', 'K', 'Q', 'J' };
  
  
  private Card deck[]   = new Card[ TOTAL_CARDS ];
  private int next_card = 0;
  
  private int index_of_first_game_card = 0;
  
  public Deck()
  {
    init();
    shuffle(0);
  }
  
  public void newGame()
  {
    index_of_first_game_card = next_card;
  }
  
  public Card dealOne()
  {
    if (next_card >= TOTAL_CARDS)
    {
      //
      // ok.  we reshuffle and keep going
      //
      BlackJack.output.print("Reshuffling deck...\n");
      reshuffle();
    }
    
    return deck[next_card++];
  }
  
  private void init()
  {
    int n = 0;
    for (int s = 0; s < 4; s++)
    {
      for (int f = 0; f < faces.length; f++)
      {
        deck[n++] = new Card(faces[f], suits[s]);
      }
    }
  }
  
  // Reshuffle that portion of the deck that is not currently in use.
  private void reshuffle()
  {
    int j = 0;

    // Move the in-use cards to the beginning of the deck.
    for (int i = index_of_first_game_card; i < next_card; i++)
    {
      Card t = deck[j];
      deck[j] = deck[i];
      deck[i] = t;
      j++;
    }

    shuffle(j);
    index_of_first_game_card = 0;
  }
  
  private void shuffle(int first_card) 
  {
    int j;
    int max_random = TOTAL_CARDS - first_card;
    
    for (int i = first_card; i < TOTAL_CARDS; i++)
    {
      Card t = deck[i];
      j = BlackJack.generator.generate(max_random);
      deck[i] = deck[first_card + j];
      deck[first_card + j] = t;
    }

    next_card = first_card;
  }
}
