/*
 * ExampleApplet.java
 *
 */

import java.applet.*;
import java.util.*;
import java.awt.*;
import java.awt.event.*;

/**
 * ExampleApplet: Applet that demonstrates 
 * PieChart.
 *
 * The applet creates a window that has a pretty background 
 * picture, and adds a PieChart.
 *
 */
public class ExampleApplet extends Applet
{
	
	Image background;

	public void init() {
		System.out.println("setting Layout...");
		setLayout(new FlowLayout());
		System.out.println("loading background image...");
		loadBackgroundImage();

		System.out.println("creating piechart...");
		// Create some round buttons, listeners, and add them to the panel
		PieChart pieChart = new PieChart();
		add(pieChart);

		Slice s1=pieChart.addSlice("blue", 50, Color.blue.darker());
		Slice s2=pieChart.addSlice("cyan", 40, Color.cyan.darker());
		Slice s3=pieChart.addSlice("gray", 30, Color.lightGray.darker());
		Slice s4=pieChart.addSlice("green", 20, Color.green.darker());
		Slice s5=pieChart.addSlice("yellow", 10, Color.yellow.darker());
		Slice s6=pieChart.addSlice("orange", 5, Color.orange.darker());
		Slice s7=pieChart.addSlice("pink", 4, Color.pink.darker());
		Slice s8=pieChart.addSlice("magenta", 3, Color.magenta.darker());
		Slice s9=pieChart.addSlice("white", 2, Color.white.darker());
		Slice s10=pieChart.addSlice("red", 11, Color.red.darker());

		// Create action listener
		ExampleActionListener alistener = new ExampleActionListener();
		s1.addActionListener(alistener);
		s2.addActionListener(alistener);
		s3.addActionListener(alistener);
		s4.addActionListener(alistener);
		s5.addActionListener(alistener);
		s6.addActionListener(alistener);
		s7.addActionListener(alistener);
		s8.addActionListener(alistener);
		s9.addActionListener(alistener);
		s10.addActionListener(alistener);

		// Create mouse listener
		ExampleMouseListener mlistener = new ExampleMouseListener();
		s1.addMouseListener(mlistener);
		s2.addMouseListener(mlistener);
		s3.addMouseListener(mlistener);
		s4.addMouseListener(mlistener);
		s5.addMouseListener(mlistener);
		s6.addMouseListener(mlistener);
		s7.addMouseListener(mlistener);
		s8.addMouseListener(mlistener);
		s9.addMouseListener(mlistener);
		s10.addMouseListener(mlistener);
	}

	public void loadBackgroundImage() {
		background = getImage(getCodeBase(), "images/orb.gif");
	}
	
	/*
	 * override update to *not* erase the background before painting
	 */
	public void update(Graphics g) { paint(g); }
	
	/*
	 * paint the background picture, then call super.paint which
	 * will paint all contained components 
	 */
	public void paint(Graphics g) {
		// paint the background image
		g.drawImage(background, 0, 0, getSize().width, getSize().height,
			getBackground(), this);
		super.paint(g);
	}
}

	class ExampleActionListener implements ActionListener {
		
	    public ExampleActionListener() {}
		
		public void actionPerformed(ActionEvent e)
		{
			System.out.println("Button Pressed: " + e.getActionCommand());
		}
	}
	class ExampleMouseListener extends MouseAdapter
	{
		public ExampleMouseListener() {}
	
		public void mouseEntered(MouseEvent e)
		{
			Slice s = (Slice)e.getComponent();
			s.setColor(s.getColor().brighter());
		}

		public void mouseExited(MouseEvent e)
		{
			Slice s = (Slice)e.getComponent();
			s.setColor(s.getColor().darker());
	    }
	}
