
import java.awt.Color;
import java.awt.Window;

public class JavaMethod
{
	// slice for this method. (so we don't have to look in up in parent)
	Slice slice;

	// piechart for this method.
	// even if it is not visible, we use it to store line number stuff.
	PieChart pc;

	// optional window for the piechart.
	Window w;

	// color iterator
	ColorIterator coloriter;

	//
	// Constructors
	//
	public JavaMethod(String methodName, Color color)
	{
		this(methodName, color, new PieChart(methodName, true));
	}
	public JavaMethod(String methodName, Color color, PieChart pie)
	{
		slice = new Slice(methodName, color);
		coloriter = new ColorIterator();
		pc = pie;
	}

	public int getIndex() {return slice.getIndex();}

	// adds a hit at the corresponding line number.
	// returns null if slice for this _method_ already exists, the slice otherwise.
	public synchronized Slice addHit(int lineNum)
	{
		// disable the paint while we add / swap...
		pc.disablePaint();

		// find the line number in the pieChart.
		Slice mySlice = (Slice) pc.getSlice(Integer.toString(lineNum));

		if (mySlice == null) {
			Slice s = new Slice(Integer.toString(lineNum), 1, coloriter.nextColor());
			pc.addSlice(s);
		} else {

			// increment it.
			pc.incSlice(mySlice);

			// now bubble it up if necessary.
			int where = mySlice.getIndex();
			while (where > 1) {
				Slice higherSlice = (Slice) pc.getSlice(where-1);
				if (higherSlice == null) break;
				if (mySlice.getValue() > higherSlice.getValue()) {
					// swap
					pc.swap(where, --where);
				} else break;
			}
		}

		// enable the paint
		pc.enablePaint();

		// increment the method slice
		// the caller must be sure to update the totals for the slice's parent piechart,
		// and also (maybe) add the slice returned to the piechart.
		if (slice==null) {
			slice = new Slice(pc.getName(), 1, coloriter.nextColor());
			return slice;
		} else {
			slice.incValue();
			return null;
		}

	} // addHit

	public /*synchronized*/ Slice getSlice()	{return slice;}
	public PieChart getPieChart() {return pc;}

	public /*synchronized*/ void clearSlice()	{slice=null;}

	public Window getWindow() {return w;}
	public void setWindow(Window win) {w=win;}

	public String getName() { return slice.getLabel(); }
	public int getValue() {return slice.getValue();}

	public String toString() { return getName(); }
}
