
import jx.EventListener;
import java.util.EventObject;
import jx.profiler.*;
import java.awt.Color;
import java.util.Enumeration;
import java.util.Hashtable;

/*
 * JavaUpdater
 *
 * This class handles updating the stored Java information.
 * It contains a handle to a piechart which is updated as the hashtable updates.
 *
 */
public class JavaUpdater implements EventListener
{

	// the hashtable contains the stored profile data.
	// it is keyed on method name. the values are "JavaMethod" objects.
	Hashtable hash;

	// note: perhaps a hash table is not necessary. perhaps an ordered
	// list would be fine, because we could assume that because a method
	// has had more hits in the past, it will continue to have more hits
	// and therefore will be near the top of the list. and besides, we
	// have to keep track of an ordered list anyway, for the piechart...
	// of course, to do this, we would have to change Slice to add some
	// way to store/retrieve the extra information (JavaMethod object)

	// these are the things that we want to update realtime.
	PieChart pieChart;

	ColorIterator coloriter;

	JavaUpdater ()
	{
		hash = new Hashtable(100);
		coloriter = new ColorIterator();
	}

	public synchronized void clearAllData()
	{
		for (Enumeration e=hash.elements(); e.hasMoreElements(); ) {
			JavaMethod jm = (JavaMethod) e.nextElement();
			jm.clearSlice();
		}
	}
	public void setPieChart(PieChart pc) { pieChart=pc; }
	public PieChart getPieChart() { return pieChart; }

	public JavaMethod getJavaMethodFromSlice(Slice s)
	{
		for (Enumeration e=hash.elements(); e.hasMoreElements(); ) {
			JavaMethod jm = (JavaMethod) e.nextElement();
			if (s == jm.getSlice()) return jm;
		}
		return null;
	}

	// receives an event every time we get a point in Java space.
	// the source of the event is simply the ProfilePoint structure.
	public synchronized void processEvent(EventObject eo)
	{
		ProfilePoint pp = (ProfilePoint)eo.getSource();

		// just double check ...
		if (!pp.isJavaMethod()) {
			System.err.println("JavaUpdater got a non-Java profile point!");
			return;
		}

		// look up the method name in the hash table.
		JavaMethod jm = (JavaMethod) hash.get(pp.methodName);
		if (jm==null) {
			// its not there, we need to add it.
			jm = new JavaMethod(pp.methodName, coloriter.nextColor());
			hash.put(pp.methodName, jm);

			/*** NOTE: let's generalize this, don't make it dependent on PieChart ***/
			// put it in the pieChart if it exists.
			if (pieChart != null) {
				pieChart.addSlice(jm.getSlice());	// assume adds at the end.
			}
		}

		Slice s = jm.addHit(pp.lineNum);
		if (pieChart != null) {
			pieChart.disablePaint();
			if (s == null) {
				pieChart.totalValue++;	// this is kind of cheating.
				pieChart.dirty=true;
			} else pieChart.addSlice(s);

			// we need to update the order now.
			int where = jm.getIndex();
			try {
				while (where > 1) {
					// find out whether this list element should move up in the list, and how far.
					Slice higherSlice = (Slice) pieChart.getSlice(where-1);
					if (higherSlice == null) break;
					if (jm.getValue() > higherSlice.getValue()) {
						// swap
						pieChart.swap(where, --where);
					} else break;
				} // while
			} catch (ArrayIndexOutOfBoundsException e) {}
			pieChart.enablePaint();
			//System.out.println("slice: "+jm.getName()+" val: "+jm.getValue()+" loc: "+where);
			return;
		} // if
	} // processEvent 
}
