
import jx.EventListener;
import jx.EventMulticaster;
import jx.profiler.*;
import java.util.EventObject;
import java.net.UnknownHostException;
import java.io.IOException;

import java.awt.Color;

public class ProfileUpdater implements Runnable
{
	Profiler profiler;
	protected EventListener listen = null;

	// listeners for particular point types:
	protected EventListener javaListen = null;
	protected EventListener vmListen = null;
	protected EventListener nativeListen = null;

	public boolean stopNotice;	// can be used instead of Thread.stop() or Thread.interrupt()
	public int period;

	ProfilePoint[] data;

	ProfileUpdater(Profiler profiler)
	{
		this.profiler = profiler;
	}

	// for profile update events
	public void addListener(EventListener listen)
	{
		this.listen = EventMulticaster.add(this.listen, listen);
	}
	public void removeListener(EventListener listen)
	{
		this.listen = EventMulticaster.remove(this.listen, listen);
	}

	// for individual type events
	public void addJavaListener(EventListener listen)
	{
		this.javaListen = EventMulticaster.add(this.javaListen, listen);
	}
	public void removeJavaListener(EventListener listen)
	{
		this.javaListen = EventMulticaster.remove(this.javaListen, listen);
	}
	public void addNativeListener(EventListener listen)
	{
		this.nativeListen = EventMulticaster.add(this.nativeListen, listen);
	}
	public void removeNativeListener(EventListener listen)
	{
		this.nativeListen = EventMulticaster.remove(this.nativeListen, listen);
	}
	public void addVMListener(EventListener listen)
	{
		this.vmListen = EventMulticaster.add(this.vmListen, listen);
	}
	public void removeVMListener(EventListener listen)
	{
		this.vmListen = EventMulticaster.remove(this.vmListen, listen);
	}

	public void broadcast(EventObject e)
	{
		if (listen != null)
			listen.processEvent(e);
	}

	// thread which updates profile data every once in a while
	public void run()
	{
		try {
			stopNotice = false;

			// send event that we are attempting to connect
			if (listen != null)
				listen.processEvent(
					new ProfileEvent(this, ProfileEvent.ATTEMPTING_TO_CONNECT,
									 profiler.getFullName()));


			// attempt to connect
			if (!profiler.open()) {

				// send failure to connect event
				if (listen != null)
					listen.processEvent(
						new ProfileEvent(this, ProfileEvent.CONNECT_FAILED,
										 "problem opening I/O port"));
				return;
			}
		}
		catch (UnknownHostException x) {
			// send failure to connect event.
			if (listen != null)
				listen.processEvent(
					new ProfileEvent(this, ProfileEvent.CONNECT_FAILED,
									 "unknown host"));
			return;
		}
		catch (IOException x) {
			// send failure to connect event.
			if (listen != null)
				listen.processEvent(
					new ProfileEvent(this, ProfileEvent.CONNECT_FAILED,
									 "cannot connect to host"));
			return;
		}

		// send successfully connected event
		if (listen != null)
			listen.processEvent(
				new ProfileEvent(this, ProfileEvent.CONNECT_SUCCEEDED));

		try {
			// set flags
			//profiler.ignoreNative();
			//profiler.ignoreVM();
			profiler.reset();

			// start profiling
			profiler.startProfiling();

			// loop
			for (;;) {
				Thread.sleep(period);	// in milliseconds
				update();
				if (stopNotice) break;
				profiler.startProfiling();	// restart profiling (necessary for LocalProfiler)
			}
		}
		catch (IOException x) {
			// send IO exception event
			if (listen != null)
				listen.processEvent(
					new ProfileEvent(this, ProfileEvent.IO_ERROR));
		}
		catch (InterruptedException x) {}
		catch (ThreadDeath x) {}
		finally {
			// close profiler
			profiler.close();
			
			// send disconnected event
			if (listen != null)
				listen.processEvent(
					new ProfileEvent(this, ProfileEvent.DISCONNECT));

			stopNotice = false;
		}
		return;	// suicide
	}

	public synchronized void update() throws IOException
	{
		// send getting data event
		if (listen != null)
			listen.processEvent(
				new ProfileEvent(this, ProfileEvent.GET_DATA));

		// get the data
		data = profiler.getData();

		// send got data event
		if (listen != null)
			listen.processEvent(
				new ProfileEvent(this, ProfileEvent.GOT_DATA, data.length));

		// send begin analysis event
		if (listen != null)
			listen.processEvent(
				new ProfileEvent(this, ProfileEvent.BEGIN_ANALYSIS, data.length));

		// now, loop through the data, and dispatch events based on types.
		ProfilePoint pp;
		for (int i=0; i<data.length; ++i) {
			pp = data[i];
			if (pp.isNative()) {
				if (nativeListen != null)
					nativeListen.processEvent(new EventObject(pp));
				continue;
			}
			if (pp.isVM()) {
				if (vmListen != null)
					vmListen.processEvent(new EventObject(pp));
				continue;
			}
			if (pp.isJavaMethod()) {
				if (javaListen != null)
					javaListen.processEvent(new EventObject(pp));
				continue;
			}
		}
		// send begin analysis event
		if (listen != null)
			listen.processEvent(
				new ProfileEvent(this, ProfileEvent.DONE_ANALYSIS, data.length));
	}
}
