import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics;

/*
 *
 * Text
 * A lightweight component which simply displays a text string.
 *
 */
public class Text extends Component
{
	String text;

	public Text()				{ this("");			}
	public Text(String text)	{ this.text = text;	}
	public Text(String text, Color color)	{ this.text = text;	super.setForeground(color);	}
	public Text(String text, Font font)		{ this.text = text; super.setFont(font);	}
	public Text(String text, Font font, Color color) { this.text = text; super.setFont(font); super.setForeground(color); }

	public String get()					{ return text;					}
	public void set(String text)
	{
		this.text = text;
		//invalidate(); // why doesn't invalidate work? it is supposed to force a new layout...
		setSize(getPreferredSize());
		repaint();
	}

	public void setFont(Font font)
	{
		super.setFont(font);
		//invalidate(); // why doesn't invalidate work? it is supposed to force a new layout...
		setSize(getPreferredSize());
		repaint();
	}

	public void setColor(Color color)
	{
		super.setForeground(color);
		repaint();
	}

	public void paint(Graphics g) 
	{
		FontMetrics fm = getFontMetrics(getFont());
		g.drawString(text, 0, fm.getMaxAscent());
		super.paint(g);
	}
	
	public Dimension getPreferredSize()
	{
		FontMetrics fm = getFontMetrics(getFont());
		return new Dimension(fm.stringWidth(text), fm.getHeight());
	}
	public Dimension getMinimumSize()	{ return getPreferredSize(); }
}
