//
// TransCtl.h : Declaration of the CTransCtl
//
// Transparent Control OCX
// Tom Armstrong & Mark Nelson
// September 1997
//
// Much of the code in this ATL header file is generated
// by Visual C++.  We'll try to mark the parts that were
// create by Tom and Mark.  The steps needed to generate
// and fill in the code are described in the article in
// more detail.
//

#ifndef __TRANSCTL_H_
#define __TRANSCTL_H_

#include "resource.h"       // main symbols


/////////////////////////////////////////////////////////////////////////////
// CTransCtl
class ATL_NO_VTABLE CTransCtl :
  public CComObjectRootEx<CComSingleThreadModel>,
  public CComCoClass<CTransCtl, &CLSID_TransCtl>,
  public CComControl<CTransCtl>,
  public CStockPropImpl<CTransCtl, ITransCtl, &IID_ITransCtl, &LIBID_TRANSPARENTCONTROLLib>,
  public IProvideClassInfo2Impl<&CLSID_TransCtl, NULL, &LIBID_TRANSPARENTCONTROLLib>,
  public IPersistStreamInitImpl<CTransCtl>,
  public IPersistStorageImpl<CTransCtl>,
  public IQuickActivateImpl<CTransCtl>,
  public IOleControlImpl<CTransCtl>,
  public IOleObjectImpl<CTransCtl>,
  public IOleInPlaceActiveObjectImpl<CTransCtl>,
  public IViewObjectExImpl<CTransCtl>,
  public IOleInPlaceObjectWindowlessImpl<CTransCtl>,
  public IDataObjectImpl<CTransCtl>,
  public ISpecifyPropertyPagesImpl<CTransCtl>,
  //
  // Added manually.
  //
  public IPersistPropertyBagImpl<CTransCtl>,
  public IObjectSafety
{
public:
  //
  // The intialization code in the ctor puts the control in
  // a known state.  The four members initialized here control
  // various display properties.
  //
  CTransCtl()
  {
    m_bstrImageFile = "";
    m_bstrStatus = "";
    m_nReadyState = READYSTATE_UNINITIALIZED;
    m_pbBuffer = 0;
  }
  //
  // The dtor frees the image buffer if necessary
  //
  ~CTransCtl()
  {
    if ( m_pbBuffer )
      delete m_pbBuffer;
  }

DECLARE_REGISTRY_RESOURCEID(IDR_TRANSCTL)

BEGIN_COM_MAP(CTransCtl)
  COM_INTERFACE_ENTRY(ITransCtl)
  COM_INTERFACE_ENTRY(IDispatch)
  COM_INTERFACE_ENTRY_IMPL(IViewObjectEx)
  COM_INTERFACE_ENTRY_IMPL_IID(IID_IViewObject2, IViewObjectEx)
  COM_INTERFACE_ENTRY_IMPL_IID(IID_IViewObject, IViewObjectEx)
  COM_INTERFACE_ENTRY_IMPL(IOleInPlaceObjectWindowless)
  COM_INTERFACE_ENTRY_IMPL_IID(IID_IOleInPlaceObject, IOleInPlaceObjectWindowless)
  COM_INTERFACE_ENTRY_IMPL_IID(IID_IOleWindow, IOleInPlaceObjectWindowless)
  COM_INTERFACE_ENTRY_IMPL(IOleInPlaceActiveObject)
  COM_INTERFACE_ENTRY_IMPL(IOleControl)
  COM_INTERFACE_ENTRY_IMPL(IOleObject)
  COM_INTERFACE_ENTRY_IMPL(IQuickActivate)
  COM_INTERFACE_ENTRY_IMPL(IPersistStorage)
  COM_INTERFACE_ENTRY_IMPL(IPersistStreamInit)
  COM_INTERFACE_ENTRY_IMPL(ISpecifyPropertyPages)
  COM_INTERFACE_ENTRY_IMPL(IDataObject)
  COM_INTERFACE_ENTRY(IProvideClassInfo)
  COM_INTERFACE_ENTRY(IProvideClassInfo2)
   // Entries below added manually
  COM_INTERFACE_ENTRY_IMPL(IPersistPropertyBag)
  COM_INTERFACE_ENTRY(IObjectSafety)
END_COM_MAP()

//
// These properties are presisted by virtue of their
// presence here in the property map.
//
BEGIN_PROPERTY_MAP(CTransCtl)
   PROP_ENTRY( "ReadyState", DISPID_READYSTATE, CLSID_NULL )
   PROP_ENTRY( "ImageFile", 1, CLSID_NULL )
END_PROPERTY_MAP()

BEGIN_MSG_MAP(CTransCtl)
  MESSAGE_HANDLER(WM_PAINT, OnPaint)
  MESSAGE_HANDLER(WM_SETFOCUS, OnSetFocus)
  MESSAGE_HANDLER(WM_KILLFOCUS, OnKillFocus)
  MESSAGE_HANDLER(WM_ERASEBKGND, OnEraseBackground)
END_MSG_MAP()

  //
  // This windows message handler is discussed in the article.
  // Under IE 3.0, we have to trick the container into drawing
  // the background behind our control.  This is done using
  // the trick document in KB article Q165073.  Note that this
  // message handler doesn't get called under IE 4.0, VB 5.0,
  // or other up-to-date containers.
  //
  LRESULT OnEraseBackground( UINT,
                             WPARAM wParam,
                             LPARAM lParam,
                             BOOL& bHandled )
  {
    ATLTRACE( _T( "Erase Background\n" ) );
    HDC dc = (HDC) wParam;
    HWND hParent = GetParent();
    int color_bits = ::GetDeviceCaps( dc, BITSPIXEL );
    HPALETTE old_pal = 0;
    if ( color_bits == 8 ) {
      HPALETTE hpal;
      if ( GetAmbientPalette( hpal ) == S_OK ) {
        old_pal = ::SelectPalette( dc, hpal, TRUE );
        ::RealizePalette( dc );
      } else
        old_pal = 0;
    }
    POINT pt;
    pt.x = 0;
    pt.y = 0;
    MapWindowPoints( hParent, &pt, 1 );
    OffsetWindowOrgEx( dc, pt.x, pt.y, &pt );
    ::SendMessage( hParent, WM_ERASEBKGND, (WPARAM) dc, 0 );
    SetWindowOrgEx( dc, pt.x, pt.y, NULL );
    if ( old_pal )
      ::SelectPalette( dc, old_pal, TRUE );
    return 0;
  }
  //
  // IViewObjectEx
  //
  STDMETHOD(GetViewStatus)(DWORD* pdwStatus)
  {
    ATLTRACE( _T("IViewObjectExImpl::GetViewStatus\n" ) );
    *pdwStatus = 0;
    return S_OK;
  }

  //
  // To use a control with Internet Explorer requires
  // the implementation of a couple of safety interfaces.
  // Basically we are attesting to the fact that we don't
  // think our control could be used for evil purposes,
  // regardless of the parametes you feed it.
  //

  //
  // IObjectSafety
  //
  STDMETHODIMP GetInterfaceSafetyOptions( REFIID riid,
                                          DWORD *pdwSupportedOptions,
                                          DWORD *pdwEnabledOptions )
  {
    *pdwSupportedOptions = INTERFACESAFE_FOR_UNTRUSTED_CALLER | INTERFACESAFE_FOR_UNTRUSTED_DATA;
    *pdwEnabledOptions = *pdwSupportedOptions;
    return S_OK;
  }
  STDMETHODIMP SetInterfaceSafetyOptions( REFIID riid,
                                          DWORD dwOptionsSetMask,
                                          DWORD dwEnabledOptions )
  {
    return S_OK;
  }
  //
  // This is the callback routine that is called during
  // the download process.  Note how we need to make use
  // of the grfBSCF value to determine where we are in
  // the download process.  This value doesn't get passed
  // by the standard implementation of CBindStatusCallback,
  // which is why we had to create our own customized
  // version using cut-and-paste code reuse.
  //
  void OnData( COurBindStatusCallback<CTransCtl>* pbsc,
               DWORD grfBSCF,
               BYTE* pBytes,
               DWORD dwSize )
  {
    ATLTRACE(_T("OnData called\n"));
    //
    // If this is the first piece of data
    // make sure the buffer is empty
    //
    if ( BSCF_FIRSTDATANOTIFICATION & grfBSCF ) {
      if ( m_pbBuffer ) {
        delete m_pbBuffer;
        m_pbBuffer = 0;
      }
    }

    // If we actually received some data
    // append it to our buffer
    if ( dwSize != 0 ) {
      // If dwTotalRead is valued, it indicates
      // that this is not the first chunk, so we
      // apend it
      if ( pbsc->m_dwTotalRead ) {
        BYTE* pbTemp = m_pbBuffer;
        m_pbBuffer = new BYTE[pbsc->m_dwTotalRead + dwSize + 1];
        memcpy( m_pbBuffer,
                pbTemp,
                pbsc->m_dwTotalRead );
        memcpy( m_pbBuffer + pbsc->m_dwTotalRead,
                pBytes,
                dwSize );
        delete [] pbTemp;
      } else {
        // the first chunk of data
        m_pbBuffer = new BYTE[dwSize + 1];
        memcpy( m_pbBuffer,
                pBytes,
                dwSize );
      }
    }
    // We have received all of the bitmap data
    // Inform the control (and user) that it
    // can now draw safely
    if ( BSCF_LASTDATANOTIFICATION & grfBSCF )
      put_ReadyState( READYSTATE_COMPLETE );
  }

// ITransCtl

public:
  STDMETHOD(get_ImageFile)(/*[out, retval]*/ BSTR *pVal);
  STDMETHOD(put_ImageFile)(/*[in]*/ BSTR newVal);
  HRESULT OnDraw(ATL_DRAWINFO& di);

  // Stock properties have to be public
  long m_nReadyState;

private:
  CComBSTR m_bstrImageFile;
  CComBSTR m_bstrStatus;

  void DrawString( HDC hdc, TCHAR* szMsg, RECT* rc );
  TCHAR m_strError[ 256 ];

  BYTE*      m_pbBuffer;
  CDib       m_dib;
};

#endif //__TRANSCTL_H_
