import jx.EventListener;
import jx.profiler.ProfilePoint;
import java.util.Hashtable;
import java.util.EventObject;
import java.awt.Color;

/*
 *
 * VMUpdater
 *
 * This is almost exactly like JavaUpdater.
 * Yet another reason why Java sucks: no templates. If there were
 * templates, there would only be one copy of this code.
 * (And, an inferface is out of the question here.)
 *
 */
public class VMUpdater implements EventListener
{
	// the hashtable contains the stored profile data.
	// it is keyed on method name. the values are Slices.
	Hashtable hash;

	// note: perhaps a hash table is not necessary. perhaps an ordered
	// list would be fine, because we could assume that because a method
	// has had more hits in the past, it will continue to have more hits
	// and therefore will be near the top of the list. and anyway, we
	// have to keep track of an ordered list anyway, for the piechart
	// and list....

	// these are the things that we want to update realtime.
	PieChart pieChart;

	ColorIterator coloriter;

	VMUpdater ()
	{
		hash = new Hashtable(100);
		coloriter = new ColorIterator();
	}

	public void clearAllData() { hash = new Hashtable(100); }
	public void setPieChart(PieChart pc) { pieChart=pc; }
	public PieChart getPieChart() { return pieChart; }

	// receives an event every time we get a point in Java space.
	// the source of the event is simply the ProfilePoint structure.
	public void processEvent(EventObject eo)
	{
		ProfilePoint pp = (ProfilePoint)eo.getSource();

		// just double check ...
		if (!pp.isVM()) {
			System.err.println("VMUpdater got a non-VM profile point!");
			return;
		}

		// look up the method name in the hash table.
		Slice vm = (Slice) hash.get(pp.methodName);
		if (vm==null) {
			// its not there, we need to add it.
			vm = new Slice(pp.methodName, coloriter.nextColor());
			hash.put(pp.methodName, vm);

			/*** NOTE: let's generalize this, don't make it dependent on PieChart ***/
			// put it in the pieChart if it exists.
			if (pieChart != null) {
				pieChart.addSlice(vm);	// assume adds at the end.
			}
		}
		if (pieChart != null) {
			pieChart.disablePaint();
			pieChart.incSlice(vm);
		}

		// we need to update the order now.
		if (pieChart != null) {
			int where = vm.getIndex();
			try {
				while (where > 1) {
					// find out whether this list element should move up in the list, and how far.
					Slice higherSlice = (Slice) pieChart.getSlice(where-1);
					if (higherSlice == null) break;
					if (vm.getValue() > higherSlice.getValue()) {
						// swap
						pieChart.swap(where, --where);
					} else break;
				} // while
			} catch (ArrayIndexOutOfBoundsException e) {}
			pieChart.enablePaint();
			return;
		} // if
	} // processEvent 
}
