package java.awt;

/*
 *
 * OrderableContainer
 *
 * This class adds "swap" and "replace" methods to java.awt.Container.
 * Why? Because otherwise, to reorder components in a Container
 * (important for lightweight components), we have to use the insert
 * and delete methods which cause the array to be copied.
 *
 */
public class OrderableContainer extends Container
{
	//public Component[] getComponents() { return component; }

	public void swap(int index1, int index2)
	{
		synchronized (Component.LOCK) {
			if ((index1 < 0) || (index1 >= ncomponents)) {
				throw new ArrayIndexOutOfBoundsException("No such child: " + index1);
			}
			if ((index2 < 0) || (index2 >= ncomponents)) {
				throw new ArrayIndexOutOfBoundsException("No such child: " + index2);
			}
			Component temp = component[index1];
			component[index1] = component[index2];
			component[index2] = temp;
		}
    }

	public void replace(int index, Component item)
	{
		synchronized (Component.LOCK) {
			if ((index < 0) || (index >= ncomponents)) {
				throw new ArrayIndexOutOfBoundsException("No such child: " + index);
			}
			component[index] = item;
		}
	}
}

