package jx;

import jx.EventListener;
import java.util.EventObject;
import java.io.Serializable;
import java.io.ObjectOutputStream;
import java.io.IOException;

public class EventMulticaster implements EventListener
{

    protected final EventListener a, b;

    protected EventMulticaster(EventListener a, EventListener b) {
		this.a = a; this.b = b;
    }

    public static EventListener add(EventListener a, EventListener b) {
        return addInternal(a, b);
    }

    public static EventListener remove(EventListener l, EventListener oldl) {
		return removeInternal(l, oldl);
    }
	
    protected EventListener remove(EventListener oldl) {
		if (oldl == a)  return b;
		if (oldl == b)  return a;
		EventListener a2 = removeInternal(a, oldl);
		EventListener b2 = removeInternal(b, oldl);
		if (a2 == a && b2 == b) {
		    return this;	// it's not here
		}
		return addInternal(a2, b2);
    }

    public void processEvent(EventObject e) {
        a.processEvent(e);
        b.processEvent(e);
    }

    protected static EventListener addInternal(EventListener a, EventListener b) {
		if (a == null)  return b;
		if (b == null)  return a;
		return new EventMulticaster(a, b);
    }

    protected static EventListener removeInternal(EventListener l, EventListener oldl) {
		if (l == oldl || l == null) {
		    return null;
		} else if (l instanceof EventMulticaster) {
		    return ((EventMulticaster)l).remove(oldl);
		} else {
		    return l;		// it's not here
		}
    }

    /* Serialization support.  
     */

    protected void saveInternal(ObjectOutputStream s, String k) throws IOException {
        if (a instanceof EventMulticaster) {
			((EventMulticaster)a).saveInternal(s, k);
        }
        else if (a instanceof Serializable) {
            s.writeObject(k);
            s.writeObject(a);
        }
        
        if (b instanceof EventMulticaster) {
			((EventMulticaster)b).saveInternal(s, k);
        }
        else if (b instanceof Serializable) {
            s.writeObject(k);
            s.writeObject(b);
        }
    }

    protected static void save(ObjectOutputStream s, String k, EventListener l) throws IOException {
		if (l == null) {
			return;
		} 
		else if (l instanceof EventMulticaster) {
			((EventMulticaster)l).saveInternal(s, k);
		}
		else if (l instanceof Serializable) {
			s.writeObject(k);
			s.writeObject(l);
		}
    }
}
