// A rectangular, mouse sensitive, region of the user's screen.
//
package jx;

import java.util.Vector;

public class GraphicsWindow
   {
   // Create a new window, initially invisible.
   // Taken: window in which to nest it (null --> parent is desktop)
   //        location of upper left corner of its frame within parent's client area
   //        size of its frame
   //
   public 
   GraphicsWindow(GraphicsWindow parent, int x, int y, int w, int h)
      {
      create(parent, x, y, w, h);
      windows.addElement(this);
      }

   // Destroy this window.
   // !!TODO: add a "finalize()" method that calls this automatically.
   //
   public void
   dispose()
      {
      windows.removeElement(this);
      delete();
      }

   // Make this window visible.
   //
   public native void 
   show();

   // --------------------------------------------------------------------------//
   // The following methods are called by the window event dispatcher.          //
   // Typically they would be overridden by subclasses to provide more specific //
   // responses to window events.                                               //
   // --------------------------------------------------------------------------//
   
   // Respond to a damaged screen image.
   // Taken:    graphics context with which to redraw the screen image
   //           (!!TODO: with its clipping information set to match the damaged region)
   // Returned: nothing
   //
   public void
   damaged(GraphicsContext gc)
      {
      }
      
   // Respond to a size change.
   // Taken:    new dimensions of this window's client area
   // Returned: nothing
   // Note:     overriding subclass methods should call this before doing any work,
   //           so size information gets recorded.
   //
   public void
   resized(int width, int height)
      {
   ///System.out.println("old w=" + this.width + " h=" + this.height);
      this.width  = width;
      this.height = height;
   ///System.out.println("new w=" + this.width + " h=" + this.height);
      }

   // Respond to a mouse button release.
   // Taken:    coordinates of mouse in this window's client area
   // Returned: nothing
   //
   public void
   released(int x, int y)
      {
      }
   
   // --------------------------------------------------------------------//
   // The following methods form the core of the window event dispatcher. //
   // --------------------------------------------------------------------//

   // Read an event from this thread's window event queue.
   // Taken:    place to put event info
   // Returned: nothing - blocks until an event appears
   // See also: dispatchEvent()
   //
   public static native void 
   readEvent(int dst[]);

   // Dispatch a window event.
   // Taken:    event info, obtained from readEvent()
   // Returned: did user click on desktop window's "close" button?
   // See also: readEvent()
   //
   public static boolean
   dispatchEvent(int eventInfo[])
      {
      int targetWindowHandle = eventInfo[0];
      int eventId            = eventInfo[1];
      int wParam             = eventInfo[2];
      int lParam             = eventInfo[3];

      // map window handle into window object
      // !!TODO: use a hash table for faster lookup?
      //
      GraphicsWindow targetWindow = null;
      for (int i = 0, n = windows.size(); i < n; ++i)
         {
         GraphicsWindow window = (GraphicsWindow)windows.elementAt(i);
         if (window.sysHandle == targetWindowHandle)
            {
            targetWindow = window;
            break;
            }
         }
	
      // map event id to window method
      //
      switch (eventId)
         {
         case 0x000F:
            {
            GraphicsContext gc = new GraphicsContext(targetWindow);
            targetWindow.damaged(gc);
            gc.dispose();
            return false;
            }

         case 0x0005:
            {
            int w =  lParam        & 0xFFFF;
            int h = (lParam >> 16) & 0xFFFF;
            targetWindow.resized(w, h);
            return false;
            }

         case 0x0202:
            {
            int x =  lParam        & 0xFFFF;
            int y = (lParam >> 16) & 0xFFFF;
            targetWindow.released(x, y);
            return false;
            }

         case 0x0401:
            {
            return true;
            }
         
         default:
            {
            System.out.println("dispatchEvent: unexpected event: " + eventId);
            return false;
            }
         }
      }
      
   // Class fields.
   //
   private static Vector windows = new Vector(); // windows allocated so far
   
   // Instance fields.
   //
   private int            sysHandle; // handle to o/s data for this window
   private GraphicsWindow parent;    // window within which this one is nested (null --> parent is desktop)
   public  int            width;     // current dimensions...
   public  int            height;    // ...of client area

   // Implementation details.
   //
   private native void    create(GraphicsWindow parent, int x, int y, int width, int height); // create o/s counterpart
   private native void    delete();                                                           // delete o/s counterpart
   }
