// Methods for JVM profiler
//
// 20 Jun 97 John Whaley
//
package jx.profiler;

// TODO: check out why the compiler won't let me synchronize on an array object ...

public class LocalProfiler implements Profiler
{

	//
	// instance variables
	//
	boolean ignoreNative = false;
	boolean ignoreVM = false;
	boolean profileAllMethods = true;
	String[] methodsToWatch;
	int numberOfMethodsToWatch = 0;
	static final int WATCH_SIZE = 10;

	//
	// constructor
	//
	public LocalProfiler()
	{
		methodsToWatch = new String [WATCH_SIZE];
	}

	//
	// methods to start/stop profiling
	//
	public native void startProfiling();	// start profiling
	public native void stopProfiling();		// stop profiling

	//
	// methods to get data in the form of program counters
	//
	public synchronized int[] getPCData()	{ synchronized (this/*.methodsToWatch*/) { return _getPCData(); } }
	public synchronized int getPC(int i)	{ synchronized (this/*.methodsToWatch*/) { return _getPC(i); } }

	//
	// methods to get data in the form of ProfilePoints
	//
	public ProfilePoint[] getData()
	{
		int[] data = getPCData();
		ProfilePoint[] points = new ProfilePoint[data.length];
		for (int i=0; i<data.length; ++i)
			points[i] = decodePC(data[i]);
		return points;
	}
	public ProfilePoint get(int i)
	{
		return decodePC(getPC(i));
	}
	//
	// method to convert a program counter to a ProfilePoint
	//
	public native ProfilePoint decodePC(int PC);

	//
	// methods to set profile data filters
	//
	public void ignoreNative()			{ ignoreNative=true;		}
	public void profileNative()			{ ignoreNative=false;		}
	public void ignoreVM()				{ ignoreVM=true;			}
	public void profileVM()				{ ignoreVM=false;			}
	public void profileAllMethods()		{ profileAllMethods=true;	}
	public void profileSelectedMethods(){ profileAllMethods=false;	}
	public boolean getIgnoreNative()	{ return ignoreNative;		}
	public boolean getIgnoreVM()		{ return ignoreVM;			}
	public boolean getProfileAllMethods(){ return profileAllMethods;}
	public synchronized void addMethod(String methodName)
	{
		synchronized (this/*.methodsToWatch*/) {
			if (numberOfMethodsToWatch < WATCH_SIZE) {
				methodsToWatch[numberOfMethodsToWatch++] = methodName;
			} else {
				// silently ignore, later we can add a resizing array.
			}
		}
	}
	public synchronized void removeMethod(String methodName)
	{
		synchronized (this/*.methodsToWatch*/) {
			int i;
			for (i=0; i<numberOfMethodsToWatch; ++i) {
				if (methodsToWatch[i] == methodName) break;
			}
			for ( ; i<numberOfMethodsToWatch-1; ++i) {
				methodsToWatch[i] = methodsToWatch[i+1];
			}
		}
		// later we can add code to shrink the array.
	}

	//
	// number of profile points collected
	//
	public synchronized int size()	{ return _size(); }

	//
	// set profiler period
	//
	public synchronized void setPeriod(int period)	{ _setPeriod(period); }
	
	//
	// reset profiler data
	//
	public synchronized void reset()	{ _reset(); }

	//
	// disposes of profiler memory
	//
	public synchronized void dispose()	{ _dispose(); }

	//
	// counts
	//
	public int getNative()	{ return 0;	}
	public int getVM()		{ return 0;	}
	public int getJava()	{ return 0;	}

	//
	// methods to open and close the connection to the VM
	//
	public boolean open()	{ return true; }
	public void close()		{ }
	public boolean valid()	{ return true; }

	//
	// returns name of machine we are connected to
	//
	public String getFullName()	{ return "local machine"; }

	//
	// private, internal methods (so that synchronization is easier.)
	//
	private native int[] _getPCData();
	private native int _getPC(int i);
	private native int _size();
	private native void _setPeriod(int period);
	private native void _reset();
	private native void _dispose();
}
