
package jx.profiler;

import java.io.IOException;
import java.net.UnknownHostException;

public interface Profiler
{

	//
	// methods to start/stop profiling
	//
	public void startProfiling() throws IOException;
	public void stopProfiling() throws IOException;

	//
	// methods to get data in the form of program counters
	//
	public int[] getPCData() throws IOException;	// get all data, reset profile pointer.
	public int getPC(int i) throws IOException;		// get one point of data

	//
	// methods to get data in the form of ProfilePoints
	//
	public ProfilePoint[] getData() throws IOException;	// get all data, reset profiler pointer.
	public ProfilePoint get(int i) throws IOException;	// get one point of data

	//
	// method to convert a program counter to a ProfilePoint
	//
	public ProfilePoint decodePC(int PC) throws IOException;

	//
	// methods to set profile data filters
	//
	public void ignoreNative() throws IOException;
	public void profileNative() throws IOException;
	public void ignoreVM() throws IOException;
	public void profileVM() throws IOException;
	public void profileAllMethods() throws IOException;
	public void profileSelectedMethods() throws IOException;
	public boolean getIgnoreNative() throws IOException;
	public boolean getIgnoreVM() throws IOException;
	public boolean getProfileAllMethods() throws IOException;
	public void addMethod(String methodName) throws IOException;
	public void removeMethod(String methodName) throws IOException;

	//
	// number of profile points collected
	//
	public int size() throws IOException;

	//
	// set profiler period
	//
	public void setPeriod(int period) throws IOException;

	//
	// reset profiler data
	//
	public void reset() throws IOException;

	//
	// disposes of profiler memory
	//
	public void dispose() throws IOException;

	//
	// totals
	//
	public int getNative() throws IOException;
	public int getVM() throws IOException;
	public int getJava() throws IOException;

	//
	// methods to open and close the connection to the VM
	//
	public boolean open()
		throws IOException, UnknownHostException;	// returns whether it was successful
	public void close();

	//
	// get the name of the computer we are connected to
	//
	public String getFullName();

	//
	// method to check connection to the VM
	//
	public boolean valid();
}
