package jx.profiler;

import java.io.*;
import java.net.*;

/*
 *
 * ProfileAnalyzer
 *
 */
public class RemoteProfiler implements Profiler
{
	//
	// Commands.
	//
	private static final int START_PROFILING =			1;
	private static final int STOP_PROFILING	=			2;
	private static final int GET_PC_DATA =				3;
	private static final int GET_PC =					4;
	private static final int GET_DATA =					5;
	private static final int DECODE_PC =				6;
	private static final int IGNORE_NATIVE =			7;
	private static final int PROFILE_NATIVE =			8;
	private static final int IGNORE_VM =				9;
	private static final int PROFILE_VM =				10;
	private static final int PROFILE_ALL_METHODS =		11;
	private static final int PROFILE_SELECTED_METHODS =	12;
	private static final int GET_IGNORE_NATIVE =		13;
	private static final int GET_IGNORE_VM =			14;
	private static final int GET_PROFILE_ALL_METHODS =	15;
	private static final int ADD_METHOD =				16;
	private static final int REMOVE_METHOD =			17;
	private static final int SIZE =						18;
	private static final int SET_PERIOD	=				19;
	private static final int RESET =					20;
	private static final int DISPOSE =					21;
	private static final int GET_NATIVE =				22;
	private static final int GET_VM =					23;
	private static final int GET_JAVA =					24;
	private static final int IGNORE_LINE_NUM =			25;
	private static final int WATCH_LINE_NUM =			26;
	private static final int GET_IGNORE_LINE_NUM =		27;

	//
	// Connection information.
	//
	String hostname;
	int port;

	//
	// Socket and stream objects.
	//
	private Socket echoSocket = null;
	private DataOutputStream os = null;
	private DataInputStream is = null;

	//
	// Constructor.
	//
	public RemoteProfiler()
	{
		this("", 6827);
	}
	public RemoteProfiler(String hostname, int port)
	{
		this.hostname = hostname;
		this.port = port;
	}

	//
	// Profiler commands.
	//
	public synchronized void startProfiling() throws IOException
	{
		if (!valid()) throw new IOException();
		os.writeByte(START_PROFILING);
	}
	public synchronized void stopProfiling() throws IOException
	{
		if (!valid()) throw new IOException();
		os.writeByte(STOP_PROFILING);
	}
	public synchronized int[] getPCData() throws IOException
	{
		if (!valid()) throw new IOException();
		os.writeByte(GET_PC_DATA);
		int size = is.readInt();
		int[] data = new int[size];
		for (int i=0; i<size; ++i) {
			data[i] = is.readInt();
		}
		return data;
	}
	public synchronized int getPC(int i) throws IOException
	{
		if (!valid()) throw new IOException();
		os.writeByte(GET_PC);
		os.writeInt(i);
		int data = is.readInt();
		return data;
	}
	public synchronized ProfilePoint[] getData() throws IOException
	{
		if (!valid()) throw new IOException();
		os.writeByte(GET_DATA);
		int high = 300;
		ProfilePoint[] data = new ProfilePoint[high];
		byte type;
		int line_num = 0;
		String methodName = null;
		int i;
		for (i=0; ; ++i) {
			type = is.readByte();
			if (type == ProfilePoint.END_OF_DATA_TYPE) break;
			if (type == ProfilePoint.JAVA_TYPE) line_num = is.readInt();
			if ((type == ProfilePoint.JAVA_TYPE) || (type == ProfilePoint.VM_TYPE)) methodName = is.readLine();
			//System.out.println("Type: "+type+"\tLine: "+line_num+"\tMethod: "+methodName);
			if (i == high) { // grow the array
				high = high << 1;
				ProfilePoint[] data2 = new ProfilePoint[high];
				System.arraycopy(data, 0, data2, 0, i);
				data = data2;
			}
			data[i] = new ProfilePoint(type, line_num, methodName);
		}
		ProfilePoint[] data2 = new ProfilePoint[i];
		System.arraycopy(data, 0, data2, 0, i);
		return data2;
	}
	public ProfilePoint get(int i) throws IOException
	{
		if (!valid()) throw new IOException();
		int pc = getPC(i);
		return decodePC(pc);
	}
	public synchronized ProfilePoint decodePC(int PC) throws IOException
	{
		if (!valid()) throw new IOException();
		os.writeByte(DECODE_PC);
		os.writeInt(PC);
		int line_num = 0;
		String methodName = null;
		byte type = is.readByte();
		if (type == ProfilePoint.JAVA_TYPE) line_num = is.readInt();
		if ((type == ProfilePoint.JAVA_TYPE) || (type == ProfilePoint.VM_TYPE)) methodName = is.readLine();
		ProfilePoint p = new ProfilePoint(type, line_num, methodName);
		return p;
	}
	public synchronized void ignoreNative() throws IOException	{ if (!valid()) throw new IOException(); os.writeByte(IGNORE_NATIVE);	}
	public synchronized void profileNative() throws IOException	{ if (!valid()) throw new IOException(); os.writeByte(PROFILE_NATIVE);	}
	public synchronized void ignoreVM() throws IOException		{ if (!valid()) throw new IOException(); os.writeByte(IGNORE_VM);		}
	public synchronized void profileVM() throws IOException		{ if (!valid()) throw new IOException(); os.writeByte(PROFILE_VM);		}
	public synchronized void profileAllMethods() throws IOException			{ if (!valid()) throw new IOException(); os.writeByte(PROFILE_ALL_METHODS);		}
	public synchronized void profileSelectedMethods() throws IOException	{ if (!valid()) throw new IOException(); os.writeByte(PROFILE_SELECTED_METHODS);	}
	public synchronized boolean getIgnoreNative() throws IOException
	{
		if (!valid()) throw new IOException();
		os.writeByte(GET_IGNORE_NATIVE);
		return (is.readByte() != 0);
	}
	public synchronized boolean getIgnoreVM() throws IOException
	{
		if (!valid()) throw new IOException();
		os.writeByte(GET_IGNORE_VM);
		return (is.readByte() != 0);
	}
	public synchronized boolean getProfileAllMethods() throws IOException
	{
		if (!valid()) throw new IOException();
		os.writeByte(GET_PROFILE_ALL_METHODS);
		return (is.readByte() != 0);
	}
	public synchronized void addMethod(String methodName) throws IOException
	{
		if (!valid()) throw new IOException();
		os.writeByte(ADD_METHOD);
		os.writeInt(methodName.length());
		os.writeBytes(methodName);
	}
	public synchronized void removeMethod(String methodName) throws IOException
	{
		if (!valid()) throw new IOException();
		os.writeByte(REMOVE_METHOD);
		os.writeInt(methodName.length());
		os.writeBytes(methodName);
	}
	public synchronized int size() throws IOException
	{
		if (!valid()) throw new IOException();
		os.writeByte(SIZE);
		return (is.readInt());
	}
	public synchronized void setPeriod(int period) throws IOException
	{
		if (!valid()) throw new IOException();
		os.writeByte(SET_PERIOD);
		os.writeInt(period);
	}
	public synchronized void reset() throws IOException
	{
		if (!valid()) throw new IOException();
		os.writeByte(RESET);
	}
	public synchronized void dispose() throws IOException
	{
		if (!valid()) throw new IOException();
		os.writeByte(DISPOSE);
	}
	public synchronized int getNative() throws IOException
	{
		if (!valid()) throw new IOException();
		os.writeByte(GET_NATIVE);
		return (is.readInt());
	}
	public synchronized int getVM() throws IOException
	{
		if (!valid()) throw new IOException();
		os.writeByte(GET_VM);
		return (is.readInt());
	}
	public synchronized int getJava() throws IOException
	{
		if (!valid()) throw new IOException();
		os.writeByte(GET_JAVA);
		return (is.readInt());
	}

	//
	// Get name of machine
	//
	public String getFullName() { return hostname+":"+Integer.toString(port); }

	//
	// Get/set hostname/port
	//
	public void setHost(String hostname) { this.hostname = hostname; }
	public void setPort(int port) { this.port = port; }
	public String getHost() { return hostname; }
	public int getPort() { return port; }

	//
	// Open and close sockets.
	//
	public synchronized boolean open() throws IOException, UnknownHostException
	{
		echoSocket = new Socket(hostname, port);
		os = new DataOutputStream(echoSocket.getOutputStream());
		is = new DataInputStream(new BufferedInputStream(echoSocket.getInputStream()));
		return valid();
	}
	public synchronized void close()
	{
		if (valid()) {
			try {
				os.close();
				is.close();
				echoSocket.close();
			} catch (IOException e) {}
		}
	}

	//
	// Returns whether socket has been opened successfully.
	//
	public synchronized boolean valid()
	{
		return (echoSocket != null && os != null && is != null);
	}

	//
	// Finalize
	//
	public void finalize() { close(); }

	//
	// Test
	//
	public static void main(String[] args)
	{
		Profiler pa;
		try {
			pa = new RemoteProfiler();

			if (pa.open()) {
				pa.startProfiling();
				for (int i=0; i<10; ++i) {
					try {Thread.sleep(1000);} catch (InterruptedException e) {}
					System.out.println(pa.size());
				}
				pa.stopProfiling();
			}

			pa.close();
		}
		catch (UnknownHostException e) {
			System.err.println("Can't find local host name");
			return;
		}
		catch (IOException e) {
			System.err.println("I/O failed on the connection");
		}
	}
}